def key_of_model(cls, *args, **kwargs):
    key_prefix = "%s.%s" % (cls.__module__, cls.__name__)

    if hasattr(cls, 'cache_version'):
        key_prefix += ".%s" % (getattr(cls, 'cache_version'))

    if args:
        if len(args) != 2:
            raise ValueError('args must be [field, val]')
        keys = "%s.%s" % (args[0], args[1])
    else:
        keys = sorted(["%s.%s" % item for item in kwargs.items()])
        keys = '.'.join(keys)
    return "%s.%s" % (key_prefix, keys)


def key_of_model_list(cls, **kwargs):
    key_prefix = "%s.%s" % (cls.__module__, cls.__name__)
    if hasattr(cls, 'list_cache_version'):
        key_prefix += ".%s" % (getattr(cls, 'list_cache_version'))

    # if len(kwargs) == 0 means i want get all
    if hasattr(cls, 'list_cache_version'):
        key_prefix += ".%s" % (getattr(cls, 'list_cache_version'))
    keys = sorted(["%s.%s" % item for item in kwargs.items()])
    keys = '.'.join(keys)
    return "%s.%s" % (key_prefix, keys)



from setuptools import setup, find_packages

setup(
    name='django-cache-extension',
    version='1.0.1',
    description='cache extension for django',
    url='https://github.com/Beeblio/django-cache-extension',
    author='Shanbay python dev group',
    author_email='developer@shanbay.com',
    license='MIT',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    packages=find_packages(exclude=('tests*',)),
    install_requires=map(lambda x: x.replace('==', '>='), open("requirements.txt").readlines()),
)



from cache_extension.cache import ExtensionCache

from django_redis.cache import RedisCache, omit_exception
from django_redis.client.default import _main_exceptions
from django_redis.exceptions import ConnectionInterrupted
from redis.exceptions import ResponseError


# use it in setting_conf/caches.py
class ExtensionRedisBackend(ExtensionCache, RedisCache):

    @omit_exception
    def incr(self, key, delta=1, version=None, client=None):
        if not client:
            client = self.client.get_client(write=True)

        key = self.make_key(key, version=version)

        try:
            try:
                value = client.incr(key, delta)
            except ResponseError:
                # if cached value or total value is greater than 64 bit signed
                # integer.
                # elif int is encoded. so redis sees the data as string.
                # In this situations redis will throw ResponseError

                # try to keep TTL of key
                timeout = client.ttl(key)
                value = self.get(key, version=version, client=client) + delta
                self.set(key, value, version=version, timeout=timeout,
                         client=client)
        except _main_exceptions as e:
            raise ConnectionInterrupted(connection=client, parent=e)

        return value
